@@ -17,7 +17,7 @@ changed_lines = 0
 for iline in range(len(lines)):
     if "astropy-iers-data" in lines[iline]:
         changed_lines += 1
-        lines[iline] = f'    "astropy-iers-data>={last_version}",\n'
+        lines[iline] = f'    "astropy-iers-data>={last_version}",'
 
 if changed_lines == 0:
     print(f"No line found containing astropy-iers-data in {project_file}")
@@ -26,4 +26,4 @@ elif changed_lines > 1:
     print(f"More than one line found containing astropy-iers-data in {project_file}")
     sys.exit(1)
 
-project_file.write_text("\n".join(lines))
+project_file.write_text("\n".join(lines) + "\n")
