@@ -1,5 +1,9 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
 import operator
+from collections.abc import Hashable, Mapping, Sequence
+from numbers import Integral
+
+import numpy as np
 
 __all__ = ["BST"]
 
@@ -25,6 +29,12 @@ class MaxValue:
     def __repr__(self):
         return "MAX"
 
+    def to_value(self, unit):
+        """Convert to a value of the given unit."""
+        # This is needed to support Quantity comparisons, in particular
+        # Quantity.searchsorted(MAX).
+        return np.float64(np.inf)
+
     __str__ = __repr__
 
 
@@ -107,7 +117,7 @@ class Node:
     __ge__ = lambda x, y: x.key >= y.key
     __gt__ = lambda x, y: x.key > y.key
     __ne__ = lambda x, y: x.key != y.key
-    __slots__ = ("key", "data", "left", "right")
+    __slots__ = ("data", "key", "left", "right")
 
     # each node has a key and data list
     def __init__(self, key, data):
@@ -172,11 +182,12 @@ class BST:
         for key, row in zip(data, row_index):
             self.add(tuple(key), row)
 
-    def add(self, key, data=None):
+    def add(self, key: tuple, data: int | None = None) -> None:
         """
         Add a key, data pair.
         """
         if data is None:
+            # nothing about this branch conforms to the IndexEngine protocol
             data = key
 
         self.size += 1
@@ -203,7 +214,7 @@ class BST:
                 curr_node.data = sorted(curr_node.data)
                 return
 
-    def find(self, key):
+    def find(self, key: tuple) -> Sequence[Integral]:
         """
         Return all data values corresponding to a given key.
 
@@ -228,14 +239,14 @@ class BST:
             return (None, None)
         return self._find_recursive(key, self.root, None)
 
-    def shift_left(self, row):
+    def shift_left(self, row: int) -> None:
         """
         Decrement all rows larger than the given row.
         """
         for node in self.traverse():
             node.data = [x - 1 if x > row else x for x in node.data]
 
-    def shift_right(self, row):
+    def shift_right(self, row: int) -> None:
         """
         Increment all rows greater than or equal to the given row.
         """
@@ -278,23 +289,23 @@ class BST:
             return self._postorder(self.root, [])
         raise ValueError(f'Invalid traversal method: "{order}"')
 
-    def items(self):
+    def items(self) -> list[tuple[Hashable, list[Integral]]]:
         """
         Return BST items in order as (key, data) pairs.
         """
         return [(x.key, x.data) for x in self.traverse()]
 
-    def sort(self):
+    def sort(self) -> None:
         """
         Make row order align with key order.
         """
         i = 0
         for node in self.traverse():
             num_rows = len(node.data)
-            node.data = [x for x in range(i, i + num_rows)]
+            node.data = list(range(i, i + num_rows))
             i += num_rows
 
-    def sorted_data(self):
+    def sorted_data(self) -> None:
         """
         Return BST rows sorted by key values.
         """
@@ -330,7 +341,7 @@ class BST:
         else:
             parent.replace(node, new_node)
 
-    def remove(self, key, data=None):
+    def remove(self, key: tuple, data: int | None = None) -> bool:
         """
         Remove data corresponding to the given key.
 
@@ -374,13 +385,13 @@ class BST:
         self.size -= 1
         return True
 
-    def is_valid(self):
+    def is_valid(self) -> bool:
         """
         Returns whether this is a valid BST.
         """
         return self._is_valid(self.root)
 
-    def _is_valid(self, node):
+    def _is_valid(self, node) -> bool:
         if node is None:
             return True
         return (
@@ -390,22 +401,31 @@ class BST:
             and self._is_valid(node.right)
         )
 
-    def range(self, lower, upper, bounds=(True, True)):
+    def range(
+        self,
+        lower: tuple[Hashable, ...] | None,
+        upper: tuple[Hashable, ...] | None,
+        bounds: tuple[bool, bool] = (True, True),
+    ) -> list[int]:
         """
         Return all nodes with keys in the given range.
 
         Parameters
         ----------
-        lower : tuple
-            Lower bound
-        upper : tuple
-            Upper bound
+        lower : tuple, None
+            Lower bound (no lower bound if None)
+        upper : tuple, None
+            Upper bound (no upper bound if None)
         bounds : (2,) tuple of bool
             Indicates whether the search should be inclusive or
             exclusive with respect to the endpoints. The first
             argument corresponds to an inclusive lower bound,
             and the second argument to an inclusive upper bound.
         """
+        if lower is None:
+            lower = (MinValue(),)
+        if upper is None:
+            upper = (MaxValue(),)
         nodes = self.range_nodes(lower, upper, bounds)
         return [x for node in nodes for x in node.data]
 
@@ -472,7 +492,7 @@ class BST:
             return -1
         return max(self._height(node.left), self._height(node.right)) + 1
 
-    def replace_rows(self, row_map):
+    def replace_rows(self, row_map: "Mapping[int, int]") -> None:
         """
         Replace all rows with the values they map to in the
         given dictionary. Any rows not present as keys in
@@ -483,5 +503,5 @@ class BST:
         row_map : dict
             Mapping of row numbers to new row numbers
         """
-        for key, data in self.items():
+        for _, data in self.items():  # noqa: PERF102
             data[:] = [row_map[x] for x in data if x in row_map]
